<?php
/**
 * @package: phpBB 3.0.10 :: AOS Notifications MOD -> root/includes/hooks/
 * @version: $Id: hook_aos_notifications.php, v 0.0.8 2012/03/06 15:56:42 Alorse Exp $
 * @copyright: Alorse <> (Fredo) http://phpbb.alorse.net/
 * @license: http://opensource.org/licenses/gpl-license.php GNU Public License
 * @author: Alorse - http://www.phpbb.com/customise/db/author/alorse/
 * 
 * */
/**
 * @ignore
 */
if (!defined('IN_PHPBB'))
{
	exit;
}

class aos_notifications
{

	var $in_aos_n;
	var $config = array();

	/**
	 * Constructor
	 */
	function aos_notifications()
	{
		/**
		 *  fix for bug: http://tracker.phpbb.com/browse/PHPBB3-9842
		 */
		if (!defined('IN_AOS_N'))
		{
			define('IN_AOS_N', true);
		}
		else
		{
			return $this->obtain_config();
		}
		
		$this->obtain_config();
	}

	/**
	 * Get config values
	 */
	function obtain_config()
	{
		global $config, $user;

		if (!isset($config['topics_per_page']	) { set_config('topics_per_page', 10	); $config['topics_per_page'] = 10; 	}
		if (!isset($config['aos_n_enable']		) { set_config('aos_n_enable', true		); $config['aos_n_enable'] = true;		}
		if (!isset($config['aos_n_vmn']			) { set_config('aos_n_vmn', true		); $config['aos_n_vmn'] = true; 		}
		if (!isset($config['aos_n_viewprofile']	) { set_config('aos_n_viewprofile', true); $config['aos_n_viewprofile'] = true;	}
		
		$num_large = ($config['topics_per_page'] % 2 == 0) ? ($config['topics_per_page'] / 2) + 3 : ($config['topics_per_page'] / 2) + 2.5;
		$this->config = array_merge($this->config, array(
			//acp
			'notifications_enable' => $config['aos_n_enable'],
			'view_more_notifications' => $config['aos_n_vmn'],
			'viewprofile' => $config['aos_n_viewprofile'],
			'noty_limit_large' => $config['topics_per_page'], // System
			'foll_limit_large' => $num_large,
			'book_limit_large' => $num_large,
			//ucp
			'ucp_an_enable' => $user->data['user_aos_n_enable'],
			'noty_limit_small' => $user->data['user_aos_n_num_show'],
			'followers_my_topics' => (!$user->data['user_aos_n_num_followers']) ? false : $user->data['user_aos_n_followers'],
			'foll_limit_small' => $user->data['user_aos_n_num_followers'],
			'bookmarks_my_topics' => (!$user->data['user_aos_n_num_followers']) ? false : $user->data['user_aos_n_bookmarks'],
			'book_limit_small' => $user->data['user_aos_n_num_bookmarks'],
			'only_my_topics' => $user->data['user_aos_n_only_my_topics'],
			'only_post_unread' => $user->data['user_aos_n_only_post_unread'],
			//default
			'user_posted_in_my_post' => true,
		));
	}

	/**
	 * @description: Initial function call
	 * 
	 * @version: 0.0.8
	 * */
	function aos_init($noty_limit = 0, $followers_limit = 0, $bookmarks_limit = 0)
	{
		global $user, $template;
		global $phpbb_root_path, $phpEx;

		// Only on regular pages
		$this->in_aos_n = (basename($user->page['page_name']) == "aos_notifications.$phpEx") ? false : true;

		$user->add_lang('mods/info_acp_aos_notifications');

		$noty_limit = (!$noty_limit) ? $this->config['noty_limit_small'] : $noty_limit;
		$followers_limit = (!$followers_limit) ? $this->config['foll_limit_small'] : $followers_limit;
		$bookmarks_limit = (!$bookmarks_limit) ? $this->config['book_limit_small'] : $bookmarks_limit;

		$total_topics = $this->total_topics();
		$total_post_unread = $this->post_unread_count();

		if ($this->config['user_posted_in_my_post'])
		{
			$this->user_posted_in_my_post($noty_limit);
		}

		if ($this->config['followers_my_topics'])
		{
			$this->look_noty_my_topics(TOPICS_WATCH_TABLE, 'my_followers', $followers_limit);
		}

		if ($this->config['bookmarks_my_topics'])
		{
			$this->look_noty_my_topics(BOOKMARKS_TABLE, 'my_bookmarks', $bookmarks_limit);
		}

		if ($this->config['only_post_unread'])
		{
			$total_topics = $total_post_unread;
		}

		$total_post_unread = $total_post_unread == 0 ? '' : $total_post_unread;

		$template->assign_vars(array(
			'BMT_ON' => $this->config['bookmarks_my_topics'],
			'FMT_ON' => $this->config['followers_my_topics'],
			'ANYLINK' => $this->in_aos_n,
			'NOTIFICATIONS_ON' => $this->config['notifications_enable'],
			'NOTIFICATIONS_UCP_ON' => $this->config['ucp_an_enable'],
			'UPIMP_ON' => $this->config['user_posted_in_my_post'],
			'VIEW_MORE_NOTIFICATIONS' => $this->config['view_more_notifications'],
			'U_NOTIFICATIONS' => append_sid("{$phpbb_root_path}aos_notifications.$phpEx"),
			'NUM_UNREAD_POST' => $this->config['user_posted_in_my_post'] ? $total_post_unread : '&nbsp;&nbsp;',
			'AN_TOTAL_TOPICS' => $total_topics,
		));
	}

	/**
	 * Displays user and themes notifications in what I posted
	 * 
	 * @return void
	 * 
	 * @version: 0.0.8
	 * */
	function user_posted_in_my_post($limit)
	{
		global $db, $template, $user, $auth;
		global $phpbb_root_path, $phpEx;

		$user_id = (int) $user->data['user_id'];

		if ($this->in_aos_n)
		{
			$start = 0;
		}
		else
		{
			$start = request_var('start', 0);
		}

		$sql_array = array(
			'SELECT' => 'tp.user_id, tp.topic_id, t.forum_id, t.topic_title, t.topic_poster, t.topic_type, t.topic_last_post_id, t.topic_last_poster_id, t.topic_last_poster_name, t.topic_last_poster_colour, t.topic_last_post_time',
			'FROM' => array(
				TOPICS_POSTED_TABLE => 'tp',
			),
			'LEFT_JOIN' => array(
				array(
					'FROM' => array(TOPICS_TABLE => 't'),
					'ON' => 'tp.topic_id = t.topic_id',
				),
			),
			'WHERE' => $db->sql_in_set('t.forum_id', array_keys($auth->acl_getf('f_read', true))) . '
			AND tp.user_id <> ' . $user_id . '
			AND t.topic_last_poster_id <> ' . $user_id . $this->only_my_topics() . '
			AND tp.topic_id = ANY(
			SELECT topic_id
				FROM ' . TOPICS_POSTED_TABLE . '
				WHERE user_id = ' . $user_id . ')',
			'GROUP_BY' => 'tp.topic_id',
			'ORDER_BY' => 't.topic_last_post_id DESC',
		);

		$topics = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($topics, $limit, $start);

		while ($row = $db->sql_fetchrow($result))
		{
			$forum_id = (int) $row['forum_id'];
			$topic_id = (int) $row['topic_id'];
			$topic_tracking_info[$forum_id] = get_complete_topic_tracking($forum_id, $topic_id, false);
			$post_unread = (isset($topic_tracking_info[$forum_id][$topic_id]) && $row['topic_last_post_time'] > $topic_tracking_info[$forum_id][$topic_id]) ? true : false;

			if (($post_unread && $this->config['only_post_unread']) || (!$this->config['only_post_unread']))
			{
				$template->assign_block_vars('notifications', array(
					'POST_AUTHOR' => $this->get_username_string($row['topic_last_poster_id'], $row['topic_last_poster_name'], $row['topic_last_poster_colour'], $this->config['viewprofile']),
					'POST_LINK' => append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $forum_id . '&amp;t=' . $topic_id . '&amp;p=' . $row['topic_last_post_id'] . "#p" . $row['topic_last_post_id']),
					'MY_TOPIC' => ($user->data['user_id'] == $row['topic_poster']) ? true : false,
					'TOPIC_TITLE' => censor_text($row['topic_title']),
					'LAST_POST_TIME' => $user->format_date($row['topic_last_post_time']),
					'MINI_POST_IMG' => ($post_unread) ? $user->img('icon_post_target_unread', 'UNREAD_POST') : $user->img('icon_post_target', 'POST'),
				));
			}
		}
		$db->sql_freeresult($result);
	}

	/**
	 * @description: The total number of notifications.
	 *
	 * @return (array) $total_topics
	 *
	 * @version: 0.0.7
	 * */
	function total_topics()
	{
		global $db, $user, $auth;

		$user_id = $user->data['user_id'];

		$sql = 'SELECT COUNT( t.topic_id ) AS topics_count
			FROM ( ' . TOPICS_POSTED_TABLE . ' tp, ' . TOPICS_TABLE . ' t)
			WHERE ' . $db->sql_in_set('t.forum_id', array_keys($auth->acl_getf('f_read', true))) . '
			AND tp.topic_id = t.topic_id
			AND t.topic_last_poster_id <> ' . $user_id . $this->only_my_topics() . '
			AND tp.user_id = ' . $user_id;
		$result = $db->sql_query($sql);
		$total_topics = (int) $db->sql_fetchfield('topics_count');
		$db->sql_freeresult($result);

		return $total_topics;
	}

	/**
	 * @description: The total number of new notifications without reading.
	 *
	 * @return (array) $total_topics
	 *
	 * @version: 0.0.8
	 * */
	function post_unread_count()
	{
		global $db, $user, $auth;

		$user_id = $user->data['user_id'];
		$post_unread_count = 0;

		$sql = 'SELECT tp.topic_id, t.forum_id, t.topic_last_post_time
			FROM ( ' . TOPICS_POSTED_TABLE . ' tp, ' . TOPICS_TABLE . ' t)
			WHERE ' . $db->sql_in_set('t.forum_id', array_keys($auth->acl_getf('f_read', true))) . '
			AND tp.topic_id = t.topic_id
			AND t.topic_last_poster_id <> ' . $user_id . $this->only_my_topics() . '
			AND tp.user_id = ' . $user_id;
		$result = $db->sql_query_limit($sql, 999);

		while ($row = $db->sql_fetchrow($result))
		{
			$forum_id = (int) $row['forum_id'];
			$topic_id = (int) $row['topic_id'];
			$topic_tracking_info[$forum_id] = get_complete_topic_tracking($forum_id, $topic_id, false);
			$post_unread = (isset($topic_tracking_info[$forum_id][$topic_id]) && $row['topic_last_post_time'] > $topic_tracking_info[$forum_id][$topic_id]) ? true : false;
			$post_unread_count += $post_unread;
		}

		$db->sql_freeresult($result);

		return $post_unread_count;
	}

	/**
	 * @description: Only for messages created by the $user.
	 *
	 * @return string
	 *
	 * @version: 0.0.7
	 * */
	function only_my_topics()
	{
		global $user;

		$user_id = (int) $user->data['user_id'];

		if ($this->config['only_my_topics'])
		{
			$only_my_topics = ' AND t.topic_poster = ' . $user_id;
		}
		else
		{
			$only_my_topics = '';
		}

		return $only_my_topics;
	}

	/**
	 * Displays users who are following my topics.
	 * Show users who marked my topics as favorites.
	 * 
	 * @return void
	 * 
	 * @version: 0.0.3
	 * */
	function look_noty_my_topics($table, $prefix, $limit)
	{
		global $db, $template, $user;
		global $phpbb_root_path, $phpEx;

		$user_id = (int) $user->data['user_id'];

		$sql_array = array(
			'SELECT' => 'tx.user_id, tx.topic_id, 
							t.forum_id, t.topic_title, 
							u.username, u.user_colour',
			'FROM' => array(
				$table => 'tx',
			),
			'LEFT_JOIN' => array(
				array(
					'FROM' => array(USERS_TABLE => 'u'),
					'ON' => 'tx.user_id = u.user_id',
				),
				array(
					'FROM' => array(TOPICS_TABLE => 't'),
					'ON' => 'tx.topic_id = t.topic_id AND tx.user_id <> t.topic_poster',
				),
			),
			'WHERE' => 't.topic_poster = ' . $user_id,
			'ORDER_BY' => 'RAND( )',
		);

		$users = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($users, $limit);
		$total_topics = 0;

		while ($row = $db->sql_fetchrow($result))
		{
			$template->assign_block_vars($prefix, array(
				'USER' => $this->get_username_string($row['user_id'], $row['username'], $row['user_colour'], $this->config['viewprofile']),
				'TOPIC_LINK' => append_sid("{$phpbb_root_path}viewtopic.$phpEx", 'f=' . $row['forum_id'] . '&amp;t=' . $row['topic_id']),
				'TOPIC_TITLE' => censor_text($row['topic_title']),
			));
			$total_topics++;
		}
		$db->sql_freeresult($result);
	}

	/**
	 * Get username details for placing into templates.
	 * 
	 * @param: (string) $colour
	 * @param: (int) $userid
	 * @param: (string) $username
	 * @return: html code
	 * 
	 * @version: 0.0.1
	 * */
	function get_username_string($userid, $username, $colour, $viewprofile)
	{
		// Verify some permissions
		if ($viewprofile)
		{
			return get_username_string('full', $userid, $username, $colour);
		}
		else
		{
			return get_username_string('no_profile', $userid, $username, $colour);
		}
	}

//	function check_attach()
//	{
//		global $db, $topic_id;
//		if ($topic_id)
//		{
//			$sql = 'SELECT topic_attachment
//			FROM ' . TOPICS_TABLE . "
//			WHERE topic_id = $topic_id";
//			$result = $db->sql_query($sql);
//			$topic_attachment = (int) $db->sql_fetchfield('topic_attachment');
//			$db->sql_freeresult($result);
//			$topic_attachment ? true : false;
//		}
//	}
//
//	function hook_exists($hook_name)
//	{
//		global $cache;
//
//		if (in_array($hook_name, $cache->obtain_hooks()))
//		{
//			return true;
//		}
//		return false;
//	}

}

function aos_notifications()
{
	global $user, $auth, $config;

	if ($user->data['user_id'] != ANONYMOUS && $auth->acl_get('u_aos_n'))
	{
		$aos = new aos_notifications();
		if (isset($aos->config['notifications_enable'])) // isset for bug: http://tracker.phpbb.com/browse/PHPBB3-9842
		{
			$aos->aos_init();
		}
	}
}

if (!defined('IN_AOS_N') && $config['aos_n_enable'])
{
	$phpbb_hook->register(array('template', 'display'), 'aos_notifications');
}
?>